import unittest
import copy
import os
import shutil
from fv3config import ConfigError
from fv3config._tables import update_diag_table_for_config
from fv3config.config.derive import (
    get_current_date,
    _get_current_date_from_coupler_res,
    _get_coupler_res_filename,
)
from fv3config._datastore import (
    get_microphysics_name,
    get_field_table_filename,
    get_diag_table_filename,
    get_data_table_filename,
)
import tempfile
from .mocks import c12_config

TEST_DIRECTORY = os.path.dirname(os.path.realpath(__file__))


DEFAULT_CONFIG = c12_config()

valid_coupler_res = """    2        (Calendar: no_calendar=0, thirty_day_months=1, julian=2, gregorian=3, noleap=4)
  2016     8     1     0     0     0        Model start time:   year, month, day, hour, minute, second
  2016     8     3     0     0     0        Current model time: year, month, day, hour, minute, second
/
"""

valid_current_date = [2016, 8, 3, 0, 0, 0]

bad_coupler_res = """    2        (Calendar: no_calendar=0, thirty_day_months=1, julian=2, gregorian=3, noleap=4)
  2016     8     1     0     0     0        Model start time:   year, month, day, hour, minute, second
  2016     8  missing  0     0     0        Current model time: year, month, day, hour, minute, second
/
"""

empty_config = {}

config_for_update_diag_table_test = {
    "experiment_name": "diag_table_test",
    "namelist": {
        "coupler_nml": {
            "current_date": valid_current_date,
            "force_date_from_namelist": True,
        }
    },
}

diag_table_test_in = "default_experiment\n2016 1 1 0 0 0\nother contents here"
diag_table_test_out = "diag_table_test\n2016 8 3 0 0 0\nother contents here"


class RunDirectory(object):
    def __init__(self, directory_path):
        os.mkdir(directory_path)
        os.mkdir(os.path.join(directory_path, "INPUT"))
        os.mkdir(os.path.join(directory_path, "RESTART"))
        self.directory_path = directory_path

    def cleanup(self):
        shutil.rmtree(self.directory_path)


class TableTests(unittest.TestCase):
    def setUp(self):
        self._run_directory_list = []

    def tearDown(self):
        for directory in self._run_directory_list:
            directory.cleanup()

    def make_run_directory(self, directory_name):
        full_path = os.path.join(TEST_DIRECTORY, directory_name)
        self._run_directory_list.append(RunDirectory(full_path))
        return full_path

    def test_default_data_table_filename(self):
        config = copy.deepcopy(DEFAULT_CONFIG)
        filename = get_data_table_filename(config)
        self.assertTrue(os.path.isfile(filename))
        self.assertTrue("data_table" in filename)

    def test_default_diag_table_filename(self):
        config = copy.deepcopy(DEFAULT_CONFIG)
        filename = get_diag_table_filename(config)
        self.assertTrue(os.path.isfile(filename))
        self.assertTrue("diag_table" in filename)

    def test_default_field_table_filename(self):
        config = copy.deepcopy(DEFAULT_CONFIG)
        filename = get_field_table_filename(config)
        self.assertTrue(os.path.isfile(filename))
        self.assertTrue("field_table" in filename)

    def test_get_specified_data_table_filename(self):
        source_rundir = self.make_run_directory("source_rundir")
        data_table_filename = os.path.join(source_rundir, "data_table")
        open(data_table_filename, "w").close()
        config = copy.deepcopy(DEFAULT_CONFIG)
        config["data_table"] = data_table_filename
        filename = get_data_table_filename(config)
        self.assertEqual(filename, data_table_filename)

    def test_get_specified_diag_table_filename(self):
        source_rundir = self.make_run_directory("source_rundir")
        diag_table_filename = os.path.join(source_rundir, "diag_table")
        open(diag_table_filename, "w").close()
        config = copy.deepcopy(DEFAULT_CONFIG)
        config["diag_table"] = diag_table_filename
        filename = get_diag_table_filename(config)
        self.assertEqual(filename, diag_table_filename)

    def test_get_bad_field_table_filename(self):
        config = copy.deepcopy(DEFAULT_CONFIG)
        config["namelist"]["gfs_physics_nml"]["imp_physics"] = -1
        with self.assertRaises(NotImplementedError):
            get_field_table_filename(config)

    def test_get_bad_microphysics_name_from_config(self):
        config = copy.deepcopy(DEFAULT_CONFIG)
        config["namelist"]["gfs_physics_nml"]["imp_physics"] = -1
        with self.assertRaises(NotImplementedError):
            get_microphysics_name(config)

    def test_get_bad_diag_table_filename(self):
        diag_table_filename = "/not/a/path/diag_table"
        config = copy.deepcopy(DEFAULT_CONFIG)
        config["diag_table"] = diag_table_filename
        with self.assertRaises(ConfigError):
            get_diag_table_filename(config)

    def test_get_diag_table_from_empty_config(self):
        with self.assertRaises(ConfigError):
            get_diag_table_filename(empty_config)

    def test_get_data_table_from_empty_config(self):
        with self.assertRaises(ConfigError):
            get_data_table_filename(empty_config)

    def test_update_diag_table_from_empty_config(self):
        rundir = self.make_run_directory("rundir")
        with self.assertRaises(ConfigError):
            update_diag_table_for_config(
                empty_config, valid_current_date, os.path.join(rundir, "source")
            )

    def test_get_current_date_from_coupler_res(self):
        rundir = self.make_run_directory("test_rundir")
        coupler_res_filename = os.path.join(rundir, "coupler.res")
        with open(coupler_res_filename, "w") as f:
            f.write(valid_coupler_res)
        current_date = _get_current_date_from_coupler_res(coupler_res_filename)
        self.assertEqual(current_date, valid_current_date)

    def test_get_current_date_from_bad_coupler_res(self):
        rundir = self.make_run_directory("test_rundir")
        coupler_res_filename = os.path.join(rundir, "coupler.res")
        with open(coupler_res_filename, "w") as f:
            f.write(bad_coupler_res)
        with self.assertRaises(ConfigError):
            _get_current_date_from_coupler_res(coupler_res_filename)

    def test_get_current_date_from_config_force_date_true(self):
        config = copy.deepcopy(DEFAULT_CONFIG)
        config["namelist"]["coupler_nml"]["force_date_from_namelist"] = True
        config["namelist"]["coupler_nml"]["current_date"] = valid_current_date
        current_date = get_current_date(config)
        self.assertEqual(current_date, valid_current_date)

    def test_get_current_date_from_config_force_date_false(self):
        config = copy.deepcopy(DEFAULT_CONFIG)
        config["namelist"]["coupler_nml"]["force_date_from_namelist"] = False
        config["namelist"]["coupler_nml"]["current_date"] = valid_current_date
        current_date = get_current_date(config)
        self.assertEqual(current_date, valid_current_date)

    def test_get_current_date_from_config_which_includes_coupler_res_asset(self):
        config = copy.deepcopy(DEFAULT_CONFIG)
        tmpdir = self.make_run_directory("test_dir")
        config["patch_files"] = {
            "source_location": tmpdir,
            "source_name": "coupler.res",
            "target_location": "INPUT",
            "target_name": "coupler.res",
            "copy_method": "copy",
        }
        with open(os.path.join(tmpdir, "coupler.res"), "w") as f:
            f.write(valid_coupler_res)
        current_date = get_current_date(config)
        self.assertEqual(current_date, valid_current_date)

    def test_get_coupler_res_filename_from_bytes_coupler_res_asset(self):
        config = copy.deepcopy(DEFAULT_CONFIG)
        config["patch_files"] = {
            "bytes": b"some data",
            "target_location": "INPUT",
            "target_name": "coupler.res",
        }
        with self.assertRaises(NotImplementedError):
            _get_coupler_res_filename(config)

    def test_get_current_date_from_config_empty_initial_conditions(self):
        config = copy.deepcopy(DEFAULT_CONFIG)
        config["initial_conditions"] = []
        config["namelist"]["coupler_nml"]["current_date"] = valid_current_date
        current_date = get_current_date(config)
        self.assertEqual(current_date, valid_current_date)

    def test_update_diag_table_for_config(self):
        rundir = self.make_run_directory("test_rundir")
        diag_table_filename = os.path.join(rundir, "diag_table")
        with open(diag_table_filename, "w") as f:
            f.write(diag_table_test_in)
        current_date = get_current_date(config_for_update_diag_table_test)
        update_diag_table_for_config(
            config_for_update_diag_table_test, current_date, diag_table_filename
        )
        with open(diag_table_filename) as f:
            self.assertEqual(diag_table_test_out, f.read())


def test_get_coupler_res_filename_from_dir():
    config = copy.deepcopy(DEFAULT_CONFIG)
    with tempfile.TemporaryDirectory() as initial_conditions_dir:
        coupler_res_filename = os.path.join(initial_conditions_dir, "coupler.res")
        with open(coupler_res_filename, "w") as f:
            f.write(valid_coupler_res)
        config["initial_conditions"] = initial_conditions_dir
        result = _get_coupler_res_filename(config)
        assert result == coupler_res_filename


if __name__ == "__main__":
    unittest.main()



import unittest
from fv3config import (
    config_to_namelist,
    config_from_namelist,
    InvalidFileError,
    ConfigError,
    enable_restart,
    DiagTable,
    DiagFieldConfig,
    DiagFileConfig,
)
import datetime
import os
from copy import deepcopy
import tempfile

from .mocks import c12_config


TEST_DIRECTORY = os.path.dirname(os.path.realpath(__file__))


DEFAULT_CONFIG = c12_config()

one_item_namelist = """&fms_io_nml
    checksum_required = .false.
/
"""
one_item_dict = {"fms_io_nml": {"checksum_required": False}}

all_types_namelist = """&my_nml
    integer_option = 1
    float_option = 2.0
    string_option = 'three'
    true_option = .true.
    false_option = .false.
/
"""
all_types_dict = {
    "my_nml": {
        "integer_option": 1,
        "float_option": 2.0,
        "string_option": "three",
        "true_option": True,
        "false_option": False,
    }
}

restart_namelist_settings = {
    "fv_core_nml": {
        "external_ic": False,
        "nggps_ic": False,
        "make_nh": False,
        "mountain": True,
        "warm_start": True,
        "na_init": 0,
    }
}

empty_dict = {}

config_with_empty_namelist = {"namelist": {}}

config_with_empty_fv_core_and_coupler_nml = {
    "namelist": {"fv_core_nml": {}, "coupler_nml": {}}
}

sample_diag_table = DiagTable(
    "sample_diag_table",
    datetime.datetime(2000, 1, 1),
    file_configs=[
        DiagFileConfig(
            "filename", 1, "days", [DiagFieldConfig("dynamics", "u", "zonal_wind")]
        )
    ],
)


class ConfigDictTests(unittest.TestCase):
    def setUp(self):
        self._run_directory_list = []

    def tearDown(self):
        for directory in self._run_directory_list:
            directory.cleanup()

    def make_run_directory(self, directory_name):
        directory = tempfile.TemporaryDirectory(directory_name)
        self._run_directory_list.append(directory)
        return directory.name

    def test_init_from_empty_namelist(self):
        rundir = self.make_run_directory("test_rundir")
        namelist_filename = os.path.join(rundir, "input.nml")
        open(namelist_filename, "a").close()
        config = config_from_namelist(namelist_filename)
        self.assertIsInstance(config, dict)

    def test_init_from_missing_namelist(self):
        rundir = self.make_run_directory("test_rundir")
        namelist_filename = os.path.join(rundir, "input.nml")
        with self.assertRaises(InvalidFileError):
            config_from_namelist(namelist_filename)

    def test_init_from_one_item_namelist(self):
        rundir = self.make_run_directory("test_rundir")
        namelist_filename = os.path.join(rundir, "input.nml")
        with open(namelist_filename, "w") as f:
            f.write(one_item_namelist)
        config = config_from_namelist(namelist_filename)
        self.assertEqual(config, one_item_dict)

    def test_init_from_many_item_namelist(self):
        rundir = self.make_run_directory("test_rundir")
        namelist_filename = os.path.join(rundir, "input.nml")
        with open(namelist_filename, "w") as f:
            f.write(all_types_namelist)
        config = config_from_namelist(namelist_filename)
        self.assertEqual(config, all_types_dict)

    def test_empty_write_to_namelist(self):
        rundir = self.make_run_directory("test_rundir")
        namelist_filename = os.path.join(rundir, "input.nml")
        config = {"namelist": {}}
        config_to_namelist(config, namelist_filename)
        self.assertTrue(os.path.isfile(namelist_filename))
        with open(namelist_filename, "r") as namelist_file:
            written_namelist = namelist_file.read()
        self.assertEqual(written_namelist, "")

    def test_one_item_write_to_namelist(self):
        rundir = self.make_run_directory("test_rundir")
        namelist_filename = os.path.join(rundir, "input.nml")
        config = {"namelist": deepcopy(one_item_dict)}
        config_to_namelist(config, namelist_filename)
        self.assertTrue(os.path.isfile(namelist_filename))
        with open(namelist_filename, "r") as namelist_file:
            written_namelist = namelist_file.read()
        self.assertEqual(written_namelist, one_item_namelist)

    def test_many_items_write_to_namelist(self):
        rundir = self.make_run_directory("test_rundir")
        namelist_filename = os.path.join(rundir, "input.nml")
        config = {"namelist": deepcopy(all_types_dict)}
        config_to_namelist(config, namelist_filename)
        self.assertTrue(os.path.isfile(namelist_filename))
        with open(namelist_filename, "r") as namelist_file:
            written_lines = namelist_file.readlines()
        target_lines = [line + "\n" for line in all_types_namelist.split("\n") if line]
        self.assertEqual(len(written_lines), len(target_lines))
        self.assertEqual(written_lines[0], target_lines[0])
        self.assertEqual(
            set(written_lines[1:6]), set(target_lines[1:6])
        )  # order doesn't matter
        self.assertEqual(written_lines[6:], target_lines[6:])

    def test_write_to_existing_namelist(self):
        rundir = self.make_run_directory("test_rundir")
        namelist_filename = os.path.join(rundir, "input.nml")
        with open(namelist_filename, "w") as f:
            f.write(one_item_namelist)
        config = {"namelist": deepcopy(all_types_dict)}
        config_to_namelist(config, namelist_filename)
        with open(namelist_filename, "r") as namelist_file:
            written_lines = namelist_file.readlines()
        target_lines = [line + "\n" for line in all_types_namelist.split("\n") if line]
        self.assertEqual(len(written_lines), len(target_lines))
        self.assertEqual(written_lines[0], target_lines[0])
        self.assertEqual(
            set(written_lines[1:6]), set(target_lines[1:6])
        )  # order doesn't matter
        self.assertEqual(written_lines[6:], target_lines[6:])

    def test_default_config_has_entries(self):
        config = DEFAULT_CONFIG.copy()
        self.assertTrue(len(config) > 0)
        self.assertIn("namelist", config)
        self.assertIsInstance(config["namelist"], dict)
        for name in ["diag_table", "data_table", "forcing"]:
            self.assertIn(name, config)
            self.assertIsInstance(config[name], str)
        for name, value in config.items():
            self.assertIsInstance(name, str, f"key {name} is not a string")


class EnableRestartTests(unittest.TestCase):
    def assert_dict_in_and_equal(self, source_dict, target_dict):
        for name, item in source_dict.items():
            self.assertIn(name, target_dict)
            if isinstance(item, dict):
                self.assert_dict_in_and_equal(item, target_dict[name])
            else:
                self.assertEqual(item, target_dict[name])

    def test_enable_restart_from_default(self):
        config = DEFAULT_CONFIG.copy()
        restart_config = enable_restart(config, "initial_conditions")
        self.assert_dict_in_and_equal(
            restart_namelist_settings, restart_config["namelist"]
        )

    def test_enable_restart_from_empty_namelist(self):
        config = enable_restart(config_with_empty_namelist, "initial_conditions")
        assert config["namelist"]["fv_core_nml"]["warm_start"]
        assert not config["namelist"]["coupler_nml"]["force_date_from_namelist"]

    def test_enable_restart_from_empty_config(self):
        with self.assertRaises(ConfigError):
            enable_restart(empty_dict, "initial_conditions")

    def test_enable_restart_makes_copy(self):
        config = DEFAULT_CONFIG.copy()
        restart_config = enable_restart(config, "initial_conditions")
        self.assertEqual(config, DEFAULT_CONFIG)
        restart_config["diag_table"] = "changed item"
        restart_config["namelist"]["fv_core_nml"]["npx"] = 0
        self.assertEqual(config, DEFAULT_CONFIG)

    def test_enable_restart_initial_conditions(self):
        config = DEFAULT_CONFIG.copy()
        restart_config = enable_restart(config, "new_path")
        self.assertEqual(restart_config["initial_conditions"], "new_path")


if __name__ == "__main__":
    unittest.main()



from mpi4py import MPI

if __name__ == "__main__":
    assert MPI.COMM_WORLD.Get_size() == 6
    with open(f"rank_{MPI.COMM_WORLD.Get_rank()}", "w") as f:
        pass  # just want to create the file



import os
import re
import uuid
import warnings

from .. import filesystem
from .._exceptions import DelayedImportError
from ._native import run_native

try:
    import kubernetes as kube
except ImportError as err:
    kube = DelayedImportError(err)


def run_kubernetes(
    config_location,
    outdir,
    docker_image,
    runfile=None,
    jobname=None,
    namespace="default",
    memory_gb=3.6,
    memory_gb_limit=None,
    cpu_count=1,
    gcp_secret=None,
    image_pull_policy="IfNotPresent",
    job_labels=None,
    submit=True,
    capture_output=True,
):
    """Submit a kubernetes job to perform a fv3run operation.

    Much of the configuration must be first saved to google cloud storage, and then
    supplied via paths to that configuration. The resulting run directory is copied
    out to a google cloud storage path. This call is non-blocking, and only submits
    a job.

    Args:
        config_location (str): google cloud storage location of a yaml file containing
            a configuration dictionary
        outdir (str): google cloud storage location to upload
            the resulting run directory
        docker_image (str): docker image name to use for execution, which has fv3config
            installed with fv3run
        runfile (str, optional): location of a python file to
            execute as the model executable, either on google cloud storage or within
            the specified docker image
        jobname (str, optional): name to use for the kubernetes job, defaults to a
            random uuid.uuid4().hex
        namespace (str, optional): kubernetes namespace for the job,
            defaults to "default"
        memory_gb (float, optional): gigabytes of memory required for the kubernetes
            worker, defaults to 3.6GB
        memory_gb_limit (float, optional): maximum memory allowed for the kubernetes
            worker, defaults to the value set by memory_gb
        cpu_count (int, optional): number of CPUs to use on the kubernetes worker
        gcp_secret (str, optional): name of kubernetes secret to mount containing a
            file ``key.json`` to use as the google cloud storage key.
        image_pull_policy (str, optional): pull policy passed on to the
            kubernetes job. if set to "Always", will always pull the latest image.
            When "IfNotPresent", will only pull if no image has already been pulled.
            Defaults to "IfNotPresent".
        job_labels (Mapping[str, str], optional): labels provided as key-value pairs
            to apply to job pod.  Useful for grouping jobs together in status checks.
        capture_output (bool, optional): If True, then the stderr and stdout
            streams will be redirected to the files `outdir/stderr.log` and `outdir/stdout.log`
            respectively.
    """

    if filesystem.is_local_path(outdir):
        warnings.warn(
            f"Output directory {outdir} is a local path, so it will not be accessible "
            "once the job finishes."
        )
    command = run_native.command(
        config_location, outdir, runfile=runfile, capture_output=capture_output
    )
    job = _get_job(
        command,
        docker_image,
        jobname,
        memory_gb,
        memory_gb_limit,
        cpu_count,
        gcp_secret,
        image_pull_policy,
        job_labels,
    )
    if submit:
        _submit_job(job, namespace)
    else:
        return job


def _get_job(
    command,
    docker_image,
    jobname=None,
    memory_gb=3.6,
    memory_gb_limit=None,
    cpu_count=1,
    gcp_secret=None,
    image_pull_policy="IfNotPresent",
    job_labels=None,
):

    kube_config = KubernetesConfig(
        jobname,
        memory_gb,
        memory_gb_limit,
        cpu_count,
        gcp_secret,
        image_pull_policy,
        job_labels,
    )
    return _create_job_object(command, docker_image, kube_config)


def _submit_job(job, namespace):
    kube.config.load_kube_config()
    api = kube.client.BatchV1Api()
    api.create_namespaced_job(body=job, namespace=namespace)


def _create_job_object(command, docker_image, kube_config):
    container = kube.client.V1Container(
        name=_get_name_from_image(docker_image),
        image=docker_image,
        image_pull_policy=kube_config.image_pull_policy,
        command=command,
        resources=kube_config.resource_requirements,
        volume_mounts=kube_config.volume_mounts,
        env=kube_config.env,
    )
    return _container_to_job(container, kube_config)


def _get_name_from_image(docker_image):
    name = os.path.basename(docker_image)
    return re.split(r"\W+", name)[0].replace("_", "-")


def _container_to_job(container, kube_config):
    labels = {"app": "fv3run"}
    labels.update(kube_config.job_labels)

    # Toleration allows operation on the bigger nodes (with the specified taint)
    toleration = kube.client.V1Toleration(
        effect="NoSchedule", key="dedicated", value="climate-sim-pool",
    )
    pod_spec = kube.client.V1PodSpec(
        restart_policy="Never",
        containers=[container],
        volumes=kube_config.volumes,
        tolerations=[toleration],
    )
    template_spec = kube.client.V1PodTemplateSpec(
        metadata=kube.client.V1ObjectMeta(labels=labels), spec=pod_spec,
    )
    job_spec = kube.client.V1JobSpec(
        template=template_spec,
        backoff_limit=0,
        completions=1,
        ttl_seconds_after_finished=100,
    )
    job = kube.client.V1Job(
        api_version="batch/v1",
        kind="Job",
        metadata=kube.client.V1ObjectMeta(name=kube_config.jobname),
        spec=job_spec,
    )
    return job


class KubernetesConfig:
    def __init__(
        self,
        jobname=None,
        memory_gb=3.6,
        memory_gb_limit=None,
        cpu_count=1,
        gcp_secret=None,
        image_pull_policy="IfNotPresent",
        job_labels=None,
    ):
        """Container for kubernetes-specific job configuration.

        Args:
            jobname (str, optional): name to use for the kubernetes job, defaults to a
                random uuid.uuid4().hex
            memory_gb (float, optional): gigabytes of memory required for the kubernetes
                worker, defaults to 3.6GB
            memory_gb_limit (float, optional): maximum memory allowed for the kubernetes
                worker, defaults to the value set by memory_gb
            cpu_count (int, optional): number of CPUs to use on the kubernetes worker
            gcp_secret (str, optional): name of kubernetes secret to mount containing a
                file ``key.json`` to use as the google cloud storage key.
            image_pull_policy (str, optional): pull policy passed on to the
                kubernetes job. if set to "Always", will always pull the latest image.
                When "IfNotPresent", will only pull if no image has already been pulled.
                Defaults to "IfNotPresent".
            job_labels (Mapping[str, str], optional): labels provided as key-value pairs
                to apply to job pod.  Useful for grouping jobs together in status checks.
        """
        if jobname is None:
            self.jobname = uuid.uuid4().hex
        else:
            self.jobname = jobname
        self.memory_gb = memory_gb
        if memory_gb_limit is None:
            self.memory_gb_limit = memory_gb
        else:
            self.memory_gb_limit = memory_gb_limit
        self.cpu_count = cpu_count
        self.gcp_secret = gcp_secret
        self.image_pull_policy = image_pull_policy
        if job_labels is not None:
            self.job_labels = job_labels
        else:
            self.job_labels = {}

    @property
    def resource_requirements(self):
        return kube.client.V1ResourceRequirements(
            limits={"memory": f"{self.memory_gb_limit:.1f}G"},
            requests={
                "memory": f"{self.memory_gb:.1f}G",
                "cpu": f"{self.cpu_count:.1f}",
            },
        )

    @property
    def _secret_volume(self):
        if self.gcp_secret is not None:
            return kube.client.V1Volume(
                name="gcp-key-secret",
                secret=kube.client.V1SecretVolumeSource(secret_name=self.gcp_secret),
            )
        else:
            return None

    @property
    def volumes(self):
        if self.gcp_secret is not None:
            return [self._secret_volume]
        else:
            return []

    @property
    def volume_mounts(self):
        if self.gcp_secret is not None:
            volume_mounts = [
                kube.client.V1VolumeMount(
                    mount_path="/secret/gcp-credentials",
                    name=self._secret_volume.name,
                    read_only=True,
                )
            ]
        else:
            volume_mounts = []
        return volume_mounts

    @property
    def env(self):
        if self.gcp_secret is not None:
            return [
                kube.client.V1EnvVar(
                    name="GOOGLE_APPLICATION_CREDENTIALS",
                    value="/secret/gcp-credentials/key.json",
                ),
                kube.client.V1EnvVar(
                    name="CLOUDSDK_AUTH_CREDENTIAL_FILE_OVERRIDE",
                    value="/secret/gcp-credentials/key.json",
                ),
            ]
        else:
            return []



import os
from ._exceptions import ConfigError

package_directory = os.path.dirname(os.path.realpath(__file__))


def update_diag_table_for_config(config, current_date, diag_table_filename):
    """Re-write first two lines of diag_table_filename with experiment_name
    and current_date from config dictionary.

    Args:
        config (dict): a configuration dictionary
        current_date (list): a list of 6 integers representing current_date
        diag_table_filename (str): diag_table filename
    """
    if "experiment_name" not in config:
        raise ConfigError("config dictionary must have a 'experiment_name' key")
    temporary_diag_table_filename = f"{diag_table_filename}_temporary"
    with open(diag_table_filename) as diag_table:
        lines = diag_table.read().splitlines()
        lines[0] = config["experiment_name"]
        lines[1] = " ".join([str(x) for x in current_date])
        with open(temporary_diag_table_filename, "w") as temporary_diag_table:
            temporary_diag_table.write("\n".join(lines))
    os.replace(temporary_diag_table_filename, diag_table_filename)



from datetime import timedelta
import copy
from .time_constants import SECONDS_IN_DAY
from .._exceptions import ConfigError


def enable_restart(config, initial_conditions):
    """Apply namelist settings for initializing from model restart files.

    Args:
        config (dict): a configuration dictionary
        initial_conditions (str): path to desired new initial conditions.

    Returns:
        dict: a configuration dictionary
    """
    if "namelist" not in config:
        raise ConfigError("config dictionary must have a 'namelist' key")
    restart_config = copy.deepcopy(config)
    restart_config["namelist"].setdefault("fv_core_nml", {})["external_ic"] = False
    restart_config["namelist"]["fv_core_nml"]["nggps_ic"] = False
    restart_config["namelist"]["fv_core_nml"]["make_nh"] = False
    restart_config["namelist"]["fv_core_nml"]["mountain"] = True
    restart_config["namelist"]["fv_core_nml"]["warm_start"] = True
    restart_config["namelist"]["fv_core_nml"]["na_init"] = 0
    restart_config["namelist"].setdefault("coupler_nml", {})[
        "force_date_from_namelist"
    ] = False
    restart_config["initial_conditions"] = initial_conditions
    return restart_config


def set_run_duration(config: dict, duration: timedelta) -> dict:
    """Set the run duration in the configuration dictionary.

    Returns a new configuration dictionary.

    Args:
        config (dict): a configuration dictionary
        duration (timedelta): the new run duration

    Returns:
        new_config (dict): configuration dictionary with the new run duration

    Raises:
        ConfigError: if the config dictionary is invalid
    """
    if "namelist" not in config:
        raise ConfigError("config dictionary must have a 'namelist' key")
    return_config = copy.deepcopy(config)
    coupler_nml = return_config["namelist"]["coupler_nml"]
    total_seconds = duration.total_seconds()
    if total_seconds % 1 != 0:
        raise ValueError("duration must be an integer number of seconds")
    coupler_nml["months"] = 0
    coupler_nml["hours"] = 0
    coupler_nml["minutes"] = 0
    days = int(total_seconds / SECONDS_IN_DAY)
    coupler_nml["days"] = days
    coupler_nml["seconds"] = int(total_seconds - (days * SECONDS_IN_DAY))
    return return_config
